package database;

import com.mongodb.client.*;
import com.mongodb.client.model.Filters;
import com.mongodb.client.model.ReplaceOptions;
import object_detection.types.ObjectSet;
import object_detection.types.Point;
import object_detection.types.PointSet;
import org.bson.Document;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MongoDBInteraction {

    private MongoClient mongoClient;
    private MongoDatabase database;
    private MongoCollection<Document> objectCollection;

    public MongoDBInteraction() {
        String uri = "mongodb+srv://Cluster03790:dlVzT2Z2bEh9@cluster03790.tk4cwyy.mongodb.net/myFirstDatabase?retryWrites=true";
        this.mongoClient = MongoClients.create(uri);
        this.database = mongoClient.getDatabase("Objects");
        this.objectCollection = database.getCollection("objectSets");
    }

    public Map<String, ObjectSet> retrieveAllObjectSets() {
        Map<String, ObjectSet> allObjectSets = new HashMap<>();
        MongoCursor<Document> cursor = objectCollection.find().iterator();
        try {
            while (cursor.hasNext()) {
                Document doc = cursor.next();
                Integer index = doc.getInteger("index");
                if (index != null) {
                    allObjectSets.put(index.toString(), convertDocumentToObjectSet(doc));
                }
            }
        } finally {
            cursor.close();
        }
        return allObjectSets;
    }

    public ObjectSet retrieveLatestObjectSet() {
        Document doc = objectCollection.find().sort(new Document("index", -1)).first();
        return doc != null ? convertDocumentToObjectSet(doc) : null;
    }

    private ObjectSet convertDocumentToObjectSet(Document doc) {
        List<Document> pointSetsDocs = doc.getList("objectSets", Document.class);
        ObjectSet objectSet = new ObjectSet();
        for (Document pointSetDoc : pointSetsDocs) {
            objectSet.objects.add(convertDocumentToPointSet(pointSetDoc));
        }
        return objectSet;
    }

    private PointSet convertDocumentToPointSet(Document doc) {
        List<Document> pointsDocs = doc.getList("points", Document.class);
        PointSet pointSet = new PointSet();
        for (Document pointDoc : pointsDocs) {
            float x = pointDoc.getDouble("x").floatValue();
            float y = pointDoc.getDouble("y").floatValue();
            float z = pointDoc.getDouble("z").floatValue();
            pointSet.addPoint(new Point(x, y, z));
        }
        return pointSet;
    }
}
